package com.flow.cmd;

import com.flow.common.core.exception.BaseException;
import lombok.AllArgsConstructor;
import org.flowable.bpmn.model.BpmnModel;
import org.flowable.bpmn.model.FlowElement;
import org.flowable.bpmn.model.MultiInstanceLoopCharacteristics;
import org.flowable.bpmn.model.UserTask;
import org.flowable.common.engine.impl.interceptor.Command;
import org.flowable.common.engine.impl.interceptor.CommandContext;
import org.flowable.engine.impl.persistence.entity.ExecutionEntity;
import org.flowable.engine.impl.persistence.entity.ExecutionEntityManager;
import org.flowable.engine.impl.util.CommandContextUtil;
import org.flowable.engine.impl.util.ProcessDefinitionUtil;

import java.util.Objects;

@AllArgsConstructor
public class GetLoopCharacteristics implements Command<MultiInstanceLoopCharacteristics> {
    private String processInstanceId;
    private String taskDefinitionKey;

    @Override
    public MultiInstanceLoopCharacteristics execute(CommandContext commandContext) {
        ExecutionEntityManager executionManager = CommandContextUtil.getExecutionEntityManager(commandContext);
        ExecutionEntity processInstance = executionManager.findById(processInstanceId);
        if (Objects.isNull(processInstance)) {
            throw new BaseException("流程不存在");
        }
        BpmnModel bpmnModel = ProcessDefinitionUtil.getBpmnModel(processInstance.getProcessDefinitionId());
        FlowElement flowElement = bpmnModel.getFlowElement(taskDefinitionKey);
        MultiInstanceLoopCharacteristics loopCharacteristics = ((UserTask) flowElement).getLoopCharacteristics();
        if (Objects.isNull(loopCharacteristics)) {
            throw new BaseException("该节点不是多实例任务");
        }
        return loopCharacteristics;
    }
}
